#!/usr/local/bin/perl -w

# perl orthSeqPicker.pl orthologTable.txt humanDB.fas

# perl orthSeqPicker.pl orthologTable.txt drosophilaDB.fas

# perl orthSeqPicker.pl orthologTable.txt cionaDB.fas


# DB は一つずつ指定してください．その度にアウトファイルに配列が足されてゆきます．
# DB を配列とハッシュに入れているため，塩基配列の DB は大きすぎて動かない可能性があります．

# Excep file をタブ区切りで保存して　orthologTable.txt を作成してください．
# HOmology Type (ortholog_one2many などの行) は無視されます．あらかじめエクセルで不要な行を削除してください．
# orthologTable.txt には，目的とする ID が必要以上入っていないか確認してください．
# とくに，ENSP と ENST を入れていたりすると，二つ配列がとれてしまいます．これはスクリーンアウトをみればわかります．

# Orthology table "020_OrthologInfo3.txt" に従って．配列を集めてくるスクリプト．
# 030_Out_Fol というフォルダに，1 行目の ID (ENS*) を名前にしたアウトファイルが作成されます．

# DB に同じ ID がいくつあるか，念のため調べてスクリーンアウトします．
#   2 つ以上あったら，手作業で一方を選択してください．
#   0 であったら，db に ID がないことを示します．


use strict;
use warnings;


my $outDir = "Out_Fol";    # 取ってきた配列を保存するフォルダ．なければ自動的に作成される．

my $orthologTable = "$ARGV[0]";
my $dataBase = "$ARGV[1]";

#my @idArray = ( );               #orthologous table の行をアレイに入れている．
my %orthTable = ( );
my $count = 0;


# orthTable の改行コードを Unix に変換する．
system ("perl -i -pe 's/\r/\n/g' $orthologTable");


open(TABLE,"$orthologTable") or die "$!";
my @orthologTable = <TABLE>;
close TABLE;

#print "$dataBases\n";
my @dataBases = split(/,/,$dataBase);    # データベース名を配列に入れる．


###### エクセルで保存した orthologous table を無名配列に入れ，%orthTable を完成させる．
# 1 列目の名前を key に，それ以外を配列とした %orthTable を作成する．
foreach my $tableLine (@orthologTable) {
  if($tableLine =~ /^ENS/){                # ENS で始まる行だけ処理．これはおもに，ヘッダー行を削除する目的．
  chomp($tableLine);
  my $clearedLine = $tableLine;
  $clearedLine =~ s/\t{2,}/	/g;
  $clearedLine =~ s/,/\t/g;
  $clearedLine =~ s/"//g;
  my @tempIDs = split(/\t/,$clearedLine);
  my $tempEnsp = shift@tempIDs;
  $orthTable{$tempEnsp} = \@tempIDs;
}
}


###### db.fas を，配列とハッシュに入れる

my @dbCont = ( );

# インファイルをあける．
open(DB,"$dataBase") or die "$!";
@dbCont = <DB>;
close DB;

# インファイルの内容が入った配列をサブルーチンに渡す．データ，NameLine，配列の長さ が帰ってくる．

my ($ref_fasHash,$ref_nameArray, $seqLength) = &fasHashReader(\@dbCont);
my %fasHash = %$ref_fasHash;
my @nameArray = @$ref_nameArray;

# ファイルのプリントアウト
#foreach (@$ref_nameArray){
#  print "$_\n$$ref_fasHash{$_}\n";
#}



###### アウトフォルダの作成の作成
print "\n#### Output folder\n";
if(-e $outDir){
  print " $outDir was found.\n\n";
} else {
  print " $outDir was made.\n\n";
  system("mkdir $outDir");
}


# アウトファイルの作成
print "\n#### Output files\n";

#  アウトプットとなる ensp ファイルの作成，あれば配列が足されてゆく
foreach my $ensp (keys %orthTable) {
  $count++;       # orthTable の行数をスクリーンアウトに出すためにカウントする．

  # $ensp ファイルがある場合．これは主に orthTable で ensp の右側にある配列を持ってくる．
  if(-e "$outDir/$ensp"){
    print "$count. $outDir/$ensp was found:  ";

    # 無名配列を開けて，db に配列があるかひとつづつ調べる．
    foreach my $id (@{$orthTable{$ensp}}) {
      my @items = grep(/$id/,@nameArray);

      if(@items > 0){                       # id が DB ある場合だけ，配列を持ってきてアウトプットファイルに付け足す．
        print " $id: ".@items." records were found in $dataBase  ";   # db 内部に $id number がどれぐらいあるか，調べる．2 個以上あったらまずい．
        # @items に入っている id を一つづつ fasHashから持ってくる．
        open(ADD,">>$outDir/$ensp");
        foreach my $fasNameLine (@items) {
          chomp($fasNameLine);
          print ADD "$fasNameLine\n$fasHash{$fasNameLine}\n";
        }
        close ADD;
      }
    }
    print "\n";


  # $ensp ファイルがない場合
  } else {
    print "$count. $outDir/$ensp was made.  ";
    
    # 1 行目の id ($orthHash の key)を検索．
    my @items = grep(/$ensp/,@nameArray);
    if (@items > 0) {
      print " $ensp: ".@items." records were found in $dataBase.  ";   # db 内部に $ensp number がどれぐらいあるか，調べる．2 個以上あったらまずい．
      open(OUT,">$outDir/$ensp");
      foreach my $fasNameLine (@items) {
        chomp($fasNameLine);
        print OUT "$fasNameLine\n$fasHash{$fasNameLine}\n";
      }
      close OUT;
    }

    # 2 行目以降の id ($orthHash の value)を検索．
    foreach my $id (@{$orthTable{$ensp}}) {
      my @items = grep(/$id/,@nameArray);

      if(@items > 0){                       # id が DB ある場合だけ，配列を持ってきてアウトプットファイルに付け足す．
        print " $id: ".@items." records were found in $dataBase  ";   # db 内部に $id number がどれぐらいあるか，調べる．2 個以上あったらまずい．
        # @items に入っている id を一つづつ fasHashから持ってくる．
        open(ADD,">>$outDir/$ensp");
        foreach my $fasNameLine (@items) {
          chomp($fasNameLine);
          print ADD "$fasNameLine\n$fasHash{$fasNameLine}\n";
        }
        close ADD;
      }
    }


    print "\n";
    
    
  }
}

print "\n";
exit;




#### サブルーチン ####

#  Fasta 形式をハッシュに読み込む Subroutine
sub fasHashReader
{
  my $name = "";
  my $seq = "";
  my $seqLength = "";
  my $stone = 0;
  my @nameArraySR = ( );
  my %fasHashSR = ( );

  my ($inFileContSR) = @_;

  foreach my $inFileLine (@$inFileContSR) {
    chomp($inFileLine);
    if($inFileLine =~ /^>/){  
      if($stone == 0){$name = $inFileLine;
        $stone = 1;  
      } else {
        $fasHashSR{$name} = $seq;  
        push(@nameArraySR,$name);
        $name = "$inFileLine";
        $seq = "";
      }
    } else {
      $seq .= $inFileLine; 
    }
  }
  $fasHashSR{$name} = $seq;
  $seqLength = length($seq);
  push(@nameArraySR,$name);

  # ハッシュと配列をメインプログラムに返す．
  return (\%fasHashSR,\@nameArraySR,\$seqLength);

}